@@ -1,7 +1,7 @@
 #ifndef MATRIX_PEDIGREE_H
 #define MATRIX_PEDIGREE_H
 
-#include "Syms.h"
+#include "lme4_utils.h"
 #include <Rdefines.h>
 
 SEXP pedigree_chol(SEXP x, SEXP ans);
